from __future__ import annotations

from pydantic import BaseModel  # type: ignore


class Department(BaseModel):
    id: int
    name: str
    nameTranslated: str | None = None
    sortKey: int

    def __repr__(self) -> str:
        return f"<Department: {self.name} [{self.id}]>"



from __future__ import annotations

from datetime import datetime

from pydantic import BaseModel  # type: ignore


class LicenseSettings(BaseModel):
    hide_licensetab: str


class Config(BaseModel):
    _current_config_file: str | None = None
    accept_datasecurity: bool | None = None
    access_control_allow_credentials: bool | None = None
    access_control_allow_origins: str | None = None
    admin_ids: list[str] | None = None
    admin_mail: str | None = None
    admin_message: str | None = None
    aiAssistantAvailable: bool | None = None
    ai_description_available_generation_count: int | None = None
    ai_description_available_generation_tests: int | None = None
    ai_description_generation_count: int | None = None
    ai_description_generation_enabled: bool | None = None
    ai_description_total_generation_count: int | None = None
    allowaiassistant: bool | None = None
    allowcheckin: str | None = None
    allowedcals: str | None = None
    allowedclients: str | None = None
    allowedresources: str | None = None
    allowedservices: str | None = None
    allowedstations: str | None = None
    allowedsyncconnections: str | None = None
    allowedsyncjobs: str | None = None
    alloweduser: str | None = None
    allowfinance: str | None = None
    allowldap: str | None = None
    allowoptigemsync: str | None = None
    allowsync: str | None = None
    alpha_book_affiliate_id: str | None = None
    alpha_book_enabled: bool | None = None
    app_security_request: bool | None = None
    authorized_persons: str | None = None
    brand: str | None = None
    build: str | None = None
    ccli_auto_reporting_enabled: bool | None = None
    ccli_last_token_refresh: str | None = None
    ccli_refresh_token: str | None = None
    chatServer: str | None = None
    chrome_active: str | None = None
    chrome_binary: str | None = None
    churchcal_active: bool | None = None
    churchcal_css: str | None = None
    churchcal_entries_last_days: int | None = None
    churchcal_firstdayinweek: int | None = None
    churchcal_maincalname: str | None = None
    churchcal_name: str | None = None
    churchcal_name_default: str | None = None
    churchcal_sortcode: int | None = None
    churchchat_allow_event_chat: bool | None = None
    churchchat_allow_group_chat: bool | None = None
    churchchat_allow_person_chat: bool | None = None
    churchchat_delete_event_chat_after_x_days: int | None = None
    churchchat_invite_ct_event_chat: bool | None = None
    churchchat_invite_ct_group_chat: bool | None = None
    churchchat_name: str | None = None
    churchchat_name_default: str | None = None
    churchchat_sortcode: int | None = None
    churchchat_start_event_chat_before_x_days: int | None = None
    churchchat_start_event_chat_for_calendars: str | None = None
    churchchat_sync_user_id: int | None = None
    churchcheckin_active: bool | None = None
    churchcheckin_label_child: str | None = None
    churchcheckin_label_parent: str | None = None
    churchcheckin_label_standard: str | None = None
    churchcheckin_name: str | None = None
    churchcheckin_name_default: str | None = None
    churchcheckin_sortcode: int | None = None
    churchcheckin_tags: str | None = None
    churchcustommodule_active: str | None = None
    churchcustommodule_name: str | None = None
    churchcustommodule_name_default: str | None = None
    churchdb_active: bool | None = None
    churchdb_archivedeletehistory: bool | None = None
    churchdb_birthdaylist_station: str | None = None
    churchdb_birthdaylist_status: str | None = None
    churchdb_cleverreach_client_id: str | None = None
    churchdb_cleverreach_client_secret: str | None = None
    churchdb_cleverreach_connected: bool | None = None
    churchdb_emailseparator: str | None = None
    churchdb_groupnotchoosable: int | None = None
    churchdb_home_lat: str | None = None
    churchdb_home_lng: str | None = None
    churchdb_mailchimp_apikey: str | None = None
    churchdb_mailchimp_connected: bool | None = None
    churchdb_mailjet_apikey: str | None = None
    churchdb_mailjet_apisecret: str | None = None
    churchdb_mailjet_connected: bool | None = None
    churchdb_memberlist_station: str | None = None
    churchdb_memberlist_status: str | None = None
    churchdb_name: str | None = None
    churchdb_name_default: str | None = None
    churchdb_sendgroupmails: bool | None = None
    churchdb_smspromote_apikey: str | None = None
    churchdb_sortcode: int | None = None
    churchfinance_active: bool | None = None
    churchfinance_name: str | None = None
    churchfinance_name_default: str | None = None
    churchfinance_sortcode: int | None = None
    churchgroup_active: bool | None = None
    churchgroup_inmenu: bool | None = None
    churchgroup_name: str | None = None
    churchgroup_name_default: str | None = None
    churchgroup_sortcode: str | None = None
    churchreport_active: bool | None = None
    churchreport_name: str | None = None
    churchreport_name_default: str | None = None
    churchreport_sortcode: int | None = None
    churchresource_active: bool | None = None
    churchresource_anonymize_for_public_user: bool | None = None
    churchresource_entries_last_days: int | None = None
    churchresource_name: str | None = None
    churchresource_name_default: str | None = None
    churchresource_send_emails: bool | None = None
    churchresource_sortcode: int | None = None
    churchservice_active: bool | None = None
    churchservice_agendashowenumeration: bool | None = None
    churchservice_ccli_token: str | None = None
    churchservice_ccli_token_secret: str | None = None
    churchservice_entries_last_days: int | None = None
    churchservice_invite_persons: bool | None = None
    churchservice_name: str | None = None
    churchservice_name_default: str | None = None
    churchservice_openservice_rememberdays: int | None = None
    churchservice_reminderhours: int | None = None
    churchservice_songwithcategoryasdir: bool | None = None
    churchservice_sortcode: int | None = None
    churchsync_active: bool | None = None
    churchsync_inmenu: bool | None = None
    churchsync_name: str | None = None
    churchsync_name_default: str | None = None
    churchsync_sortcode: str | None = None
    churchwiki_active: bool | None = None
    churchwiki_name: str | None = None
    churchwiki_name_default: str | None = None
    churchwiki_sortcode: int | None = None
    cron_daily: datetime | None = None
    cron_hour_8: datetime | None = None
    cronjob_dbdump: bool | None = None
    cronjob_delay: int | None = None
    csrf_enabled: bool | None = None
    currently_mail_sending: str | None = None
    datasecurity_banner_enabled: bool | None = None
    datasecurity_privacy_agreement_text: str | None = None
    datasecurity_privacy_agreement_text_for_children: str | None = None
    db_name: str | None = None
    db_password: str | None = None
    db_server: str | None = None
    db_user: str | None = None
    default_phone_area_code: str | None = None
    emailServer: str | None = None
    encryptionkey: str | None = None
    env: str | None = None
    evangelische_termine_api_key: str | None = None
    evangelische_termine_enabled: bool | None = None
    evangelische_termine_name: str | None = None
    evangelische_termine_url: str | None = None
    evangelische_termine_vid: str | None = None
    feature_custommodule: str | None = None
    feature_darkmode: str | None = None
    feature_group: str | None = None
    feature_sync: str | None = None
    finance_active: bool | None = None
    finance_inmenu: str | None = None
    finance_name: str | None = None
    finance_name_default: str | None = None
    finance_sortcode: str | None = None
    first_sync_job: datetime | None = None
    first_transaction: datetime | None = None
    hostingservice: str | None = None
    https_only: str | None = None
    image_extension: str | None = None
    installation_verification_code: str | None = None
    invite_email_text: str | None = None
    isPostsActive: bool | None = None
    isSamlActive: bool | None = None
    is_churchtools_blog_widget_active: bool | None = None
    is_churchtools_onboarding_widget_active: bool | None = None
    is_pr_widget_active: bool | None = None
    is_rss_widget_active: bool | None = None
    language: str | None = None
    last_cron: str | None = None
    last_cron_finished: str | None = None
    last_db_dump: str | None = None
    last_import_clear: str | None = None
    last_translation_update: str | None = None
    ldap_otp_enabled: bool | None = None
    licenseSettings: LicenseSettings | None = None
    log_debug: str | None = None
    login_message: str | None = None
    mail_enabled: bool | None = None
    mail_sending_in_background: str | None = None
    mail_sending_starttime: str | None = None
    mail_smtp_args_host: str | None = None
    mail_smtp_args_password: str | None = None
    mail_smtp_args_port: str | None = None
    mail_smtp_args_smtpsecure: str | None = None
    mail_smtp_args_username: str | None = None
    max_uploadfile_size_kb: int | None = None
    memberlist_birthday_full: str | None = None
    memberlist_email: str | None = None
    memberlist_fax: str | None = None
    memberlist_group_couples: str | None = None
    memberlist_picture: str | None = None
    memberlist_salutation: str | None = None
    memberlist_telefongeschaeftlich: str | None = None
    memberlist_telefonhandy: str | None = None
    memberlist_telefonprivat: str | None = None
    onboarding_start: datetime | None = None
    openstreetmaps_enabled: bool | None = None
    orderstatus: str | None = None
    orderstatus_since_date: datetime | None = None
    package: str | None = None
    post_active: bool | None = None
    post_edit_time_limited: bool | None = None
    post_email_summary_default_enabled: bool | None = None
    post_featured_groups: str | None = None
    post_name: str | None = None
    post_sortcode: int | None = None
    post_wizard_completed: bool | None = None
    post_wizard_groups: str | None = None
    prevent_change_security_settings: bool | None = None
    prevent_export: bool | None = None
    privacy_policy_external: bool | None = None
    privacy_policy_external_link: str | None = None
    privacy_policy_fields_mandatory: bool | None = None
    privacy_policy_fields_not_mandatory_api: bool | None = None
    privacy_policy_internal: bool | None = None
    privacy_policy_relationships: str | None = None
    profile: str | None = None
    rabbitmq_config_host: str | None = None
    rabbitmq_config_password: str | None = None
    rabbitmq_config_port: str | None = None
    rabbitmq_config_user: str | None = None
    rss_widget_link: str | None = None
    safe_mode_enable_authorized_persons: str | None = None
    safe_mode_enable_chat_sync: str | None = None
    safe_mode_enable_consolidation: str | None = None
    safe_mode_enable_guid_sync: str | None = None
    safe_mode_enable_job_queueing: str | None = None
    safe_mode_enable_mail: str | None = None
    safe_mode_enable_newsletter: str | None = None
    safe_mode_enable_notification: str | None = None
    send_data_security_mails: bool | None = None
    short_name: str | None = None
    show_remember_me: bool | None = None
    site_language: str | None = None
    site_licensekey: str | None = None
    site_logo: str | None = None
    site_mail: str | None = None
    site_name: str | None = None
    site_offline: bool | None = None
    site_startpage: str | None = None
    site_url: str | None = None
    support_user_active_since: datetime | None = None
    test: str | None = None
    timezone: str | None = None
    version: str | None = None
    webchatLink: str | None = None
    website_order_status: str | None = None
    website_sync_user_id: str | None = None
    website_trial_user_id: int | None = None
    website_url: str | None = None
    welcome: str | None = None
    welcome_subtext: str | None = None


class SearchResult(BaseModel):
    apiUrl: str
    domainIdentifier: str
    domainType: str
    frontendUrl: str
    icon: str
    imageUrl: str | None = None
    title: str

    # TODO: domainAttributes

    def __repr__(self) -> str:
        return f"<SearchResult: {self.title} [{self.domainType}]>"


class SimulateStop(BaseModel):
    redirect: str


class VersionInfo(BaseModel):
    build: str
    shortName: str
    siteName: str
    version: str

    def __repr__(self) -> str:
        return f"<Version: {self.version} (build: {self.build})>"



from __future__ import annotations

from datetime import date, datetime

from pydantic import BaseModel  # type: ignore

from . import EmptyStrToNone


class Person(BaseModel):
    id: int | None = None
    securityLevelForPerson: int | None = None
    editSecurityLevelForPerson: int | None = None
    title: str | None = None
    firstName: str | None = None
    lastName: str | None = None
    nickname: str | None = None
    job: str | None = None
    street: str | None = None
    addressAddition: str | None = None
    zip: str | None = None
    city: str | None = None
    country: str | None = None
    latitude: EmptyStrToNone[float] | None = None
    longitude: EmptyStrToNone[float] | None = None
    latitudeLoose: EmptyStrToNone[float] | None = None
    longitudeLoose: EmptyStrToNone[float] | None = None
    phonePrivate: str | None = None
    phoneWork: str | None = None
    mobile: str | None = None
    fax: str | None = None
    birthName: str | None = None
    birthday: EmptyStrToNone[date] | None = None
    birthplace: str | None = None
    imageUrl: str | None = None
    familyImageUrl: str | None = None
    sexId: int | None = None
    email: str | None = None
    cmsUserId: str | None = None
    optigemUserId: str | None = None
    nationalityId: int | None = None
    familyStatusId: int | None = None
    weddingDate: EmptyStrToNone[date] | None = None
    campusId: int | None = None
    statusId: int | None = None
    departmentIds: list[int] | None = None
    firstContact: EmptyStrToNone[datetime] | None = None
    dateOfBelonging: EmptyStrToNone[date] | None = None
    dateOfEntry: EmptyStrToNone[datetime] | None = None
    dateOfResign: EmptyStrToNone[datetime] | None = None
    dateOfBaptism: EmptyStrToNone[date] | None = None
    placeOfBaptism: str | None = None
    baptisedBy: str | None = None
    referredBy: str | None = None
    referredTo: str | None = None
    growPathId: int | None = None
    isArchived: bool | None = None

    # TODO: privacyPolicyAgreement
    # TODO: meta

    def __repr__(self) -> str:
        return f"<Person: {self.firstName} {self.lastName} [{self.id}]>"


class PersonTag(BaseModel):
    id: int
    name: str
    count: int


class PersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class PersonDomainObject(BaseModel):
    domainAttributes: PersonDomainAttributes
    imageUrl: str
    frontendUrl: str
    apiUrl: str
    domainType: str
    title: str
    domainIdentifier: str


class PersonRelationship(BaseModel):
    relative: PersonDomainObject
    degreeOfRelationship: str
    relationshipName: str
    relationshipTypeId: int

    def __repr__(self) -> str:
        return (
            f"<Relationship: {self.relative.title} " f"({self.degreeOfRelationship})>"
        )


class BirthdayPersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class BirthdayPerson(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: str | None = None
    domainAttributes: BirthdayPersonDomainAttributes

    def __repr__(self) -> str:
        return f"<BirthdayPerson: {self.title}>"


class Birthday(BaseModel):
    type: str
    date: date
    age: int | None = None
    person: BirthdayPerson

    def __repr__(self) -> str:
        return f"<Birthday: {self.person.title} {self.date} " f"({self.age} years)>"


class Device(BaseModel):
    id: str
    type: str
    ttl: datetime
    version: str
    createdAt: datetime
    updatedAt: datetime



from __future__ import annotations

from pydantic import BaseModel


class AgeGroup(BaseModel):
    end: int | None = None
    id: int
    name: str
    nameTranslated: str | None = None
    start: int | None = None
    sortKey: int | None = None


class GroupDomainAttributes(BaseModel):
    note: str


class Group(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: str | None = None
    domainAttributes: GroupDomainAttributes

    def __repr__(self) -> str:
        return f"<Group: {self.title}>"



from __future__ import annotations

from datetime import datetime

from pydantic import BaseModel  # type: ignore


class AgendaSong(BaseModel):
    songId: int
    arrangementId: int
    title: str
    arrangement: str
    category: str
    key: str
    bpm: str
    defaultArrangement: str | None = None


class AgendaItem(BaseModel):
    id: int
    position: int
    type: str
    title: str
    note: str | None = None
    duration: int
    start: datetime
    isBeforeEvent: bool
    song: AgendaSong | None = None

    # TODO: responsible
    # TODO: serviceGroupNotes
    # TODO: meta


class Agenda(BaseModel):
    id: int
    name: str
    series: str
    isFinal: bool
    calendarId: int
    total: int
    items: list[AgendaItem]

    # TODO: meta

    def __repr__(self) -> str:
        return f"<Agenda: {self.name} [{self.id}]>"


class EventService(BaseModel):
    id: int
    name: str | None = None
    serviceId: int
    agreed: bool
    isValid: bool
    requestedDate: datetime
    comment: str
    counter: int
    allowChat: bool

    # TODO: person
    # TODO: requesterPerson

    def __repr__(self) -> str:
        return f"<EventService: {self.name}>"


class Event(BaseModel):
    id: int
    guid: str
    name: str
    description: str | None = None
    startDate: datetime
    endDate: datetime
    chatStatus: str
    eventServices: list[EventService] | None = None

    # TODO: permissions
    # TODO: calendar

    def __repr__(self) -> str:
        return (
            f"<Event: {self.startDate.day:02}."
            f"{self.startDate.month:02} {self.startDate.hour:02}:"
            f'{self.startDate.minute:02} "{self.name}" [{self.id}]>'
        )



from __future__ import annotations

from pydantic import BaseModel

from churchtools.models.group import AgeGroup
from churchtools.models.person import PersonDomainObject


class GroupHomepageDomainAttributes(BaseModel):
    childGroupIds: list[int] | None = None


class GroupHomepageSimple(BaseModel):
    apiUrl: str | None = None
    domainAttributes: GroupHomepageDomainAttributes | None = None
    frontendUrl: str | None = None
    icon: str | None = None
    imageUrl: str | None = None
    title: str | None = None

    def get_hash(self) -> str:
        if self.apiUrl:
            return self.apiUrl.split("/")[-1]

        return ""


class GroupInformation(BaseModel):
    ageGroups: list[AgeGroup] | None = None
    campus: dict | None = None
    groupCategory: dict | None = None
    groupPlaces: list[dict] | None = None
    imageUrl: str | None = None
    leader: list[PersonDomainObject] | None = None
    meetingTime: str | None = None
    note: str | None = None
    targetGroup: dict | None = None
    weekday: dict | None = None


class GroupDetail(BaseModel):
    allowWaitinglist: bool | None = None
    autoAccept: bool | None = None
    canSignUp: bool | None = None
    children: list[int] | None = None
    currentMemberCount: int | None = None
    id: int | None = None
    information: GroupInformation | None = None
    maxMemberCount: int | None = None
    name: str | None = None
    requestedPlacesCount: int | None = None
    requestedWaitinglistPlacesCount: int | None = None
    signUpConditions: dict | None = None
    signUpHeadline: str | None = None
    signUpPersons: dict | None = None


class GroupHomepage(BaseModel):
    defaultView: str | None = None
    filter: list[dict] | None = None
    groups: list[GroupDetail]
    id: int | None = None
    isEnabled: bool | None = None
    meta: dict | None = None
    orderDirection: str | None = None
    parentGroup: int | None = None
    randomUrl: str | None = None
    showFilter: bool | None = None
    showGroups: bool | None = None
    showLeader: bool | None = None
    showMap: bool | None = None
    sortBy: str | None = None



from __future__ import annotations

from datetime import date, datetime

from pydantic import BaseModel

from churchtools.models.calendar import AppointmentMeta, CalendarMeta


class BookingAdditional(BaseModel):
    date: date | datetime
    id: int
    isRepeated: bool | None = None
    meta: CalendarMeta


class ResourceType(BaseModel):
    campusId: int | None = None
    id: int
    name: str
    nameTranslated: str
    sortKey: int


class Resource(BaseModel):
    adminIds: list[int] | None = None
    doesRequireCalEntry: bool
    iCalLocation: str | None = None
    id: int
    isAutoAccept: bool
    isVirtual: bool
    location: str | None = None
    name: str
    nameTranslated: str
    randomString: str
    resourceTypeId: int
    sortKey: int


class StartEndDate(BaseModel):
    startDate: datetime
    endDate: datetime


class Booking(BaseModel):
    additionals: list[BookingAdditional]
    allDay: bool
    calculated: StartEndDate | None = None
    calId: int | None = None
    caption: str
    endDate: date | datetime
    exceptions: list[BookingAdditional]
    id: int
    location: str | None = None
    meta: AppointmentMeta | None = None
    note: str | None = None
    personId: int | None = None
    repeatFrequency: int | None = None
    repeatId: int
    repeatOption: int | None = None
    repeatUntil: str | None = None
    resource: Resource
    showInCal: bool
    startDate: date | datetime
    statusId: int
    version: int



"""
Admin
=====

Admin relevant endpoints

"""

from __future__ import annotations

from datetime import datetime
from typing import Any

from churchtools.models.admin import LogEntry, LoginStatistic, SecurityLevel
from churchtools.models.pagination import MetaPagination


class Admin:
    def __init__(self, ct: Any) -> None:
        """Initialize a Persons object."""

        self.__ct = ct

    def logs(
        self,
        message: str | None = None,
        levels: list[str] | None = None,
        before: datetime | None = None,
        after: datetime | None = None,
        person_id: int | None = None,
        page: int | None = None,
        limit: int | None = None,
    ) -> tuple[list[LogEntry], MetaPagination | None]:
        """The response is a collection of all log messages you may see and is limited to a specific number of messages.
        You can use the page parameter to browse the list of log messages. The logs are ordered by date.

        :param message: Filter by message text
        :type message: str
        :param levels: Filter by log level ID
        :type levels: List[str]
        :param before: Filter log messages before that date.
        :type before: datetime
        :param after: Filter log messages after that date.
        :type after: datetime
        :param person_id: Filter by person
        :type person_id: int
        :param page: Page number to show page in pagination. If empty, start at first page.
        :type page: int
        :param limit: Number of results per page (default: 10)
        :type limit: int
        :returns: A list of log messages & pagination
        :rtype: Tuple[List[LogEntry], MetaPagination]
        """

        params: dict[str, Any] = {}

        if message:
            params["message"] = message
        if levels:
            params["levels"] = levels
        if before:
            params["before"] = before.strftime("%Y-%m-%dT%H:%M:%SZ")
        if after:
            params["after"] = after.strftime("%Y-%m-%dT%H:%M:%SZ")
        if person_id:
            params["person_id"] = person_id
        if page:
            params["page"] = page
        if limit:
            params["limit"] = limit

        res = self.__ct.make_request("logs", params=params)

        logs: list[LogEntry] = []
        pagination = None
        if res and "data" in res:
            for log in res["data"]:
                logs.append(LogEntry(**log))

        if res and "meta" in res:
            pagination = MetaPagination(**res["meta"])

        return logs, pagination

    def get_log(self, id: int) -> LogEntry | None:
        """Fetch one specific log message by its ID

        :param id: ID of the entity
        :type id: int
        :returns: The log entry
        :rtype: Optional[LogEntry]
        """

        res = self.__ct.make_request(f"logs/{id}")

        if res and "data" in res:
            return LogEntry(**res["data"])

        return None

    def login_statistics(
        self,
        order_by: str | None = None,
        page: int | None = None,
        limit: int | None = None,
    ) -> tuple[list[LoginStatistic], MetaPagination | None]:
        """Get statistics about login counts of users.

        :param order_by: Order the pagination result. Allowed values: `frequent` and `last`
        :type order_by: str
        :param page: Page number to show page in pagination. If empty, start at first page.
        :type page: int
        :param limit: Number of results per page (default: 10)
        :type limit: int
        :returns: A list of login statistics & pagination
        :rtype: Tuple[List[LogEntry], Optional[MetaPagination]]
        """

        params: dict[str, Any] = {}
        if order_by:
            params["order_by"] = order_by
        if page:
            params["page"] = page
        if limit:
            params["limit"] = limit

        res = self.__ct.make_request("logs/statistics/login", params=params)

        login_statistics: list[LoginStatistic] = []
        pagination = None
        if res and "data" in res:
            for login in res["data"]:
                login_statistics.append(LoginStatistic(**login))
        if res and "meta" in res:
            pagination = MetaPagination(**res["meta"])

        return login_statistics, pagination

    def list_security_levels(
        self,
    ) -> tuple[list[SecurityLevel], MetaPagination | None]:
        """Get all security levels

        :returns: A list of security levels & pagination
        :rtype: Tuple[List[LogEntry], Optional[MetaPagination]]
        """

        res = self.__ct.make_request("securitylevels")

        securitylevels: list[SecurityLevel] = []
        pagination = None
        if res and "data" in res:
            for level in res["data"]:
                securitylevels.append(SecurityLevel(**level))
        if res and "meta" in res:
            pagination = MetaPagination(**res["meta"])

        return securitylevels, pagination

    def create_security_level(self, id: int, name: str) -> SecurityLevel | None:
        """Create a new security level.

        :param id: The ID of the new security level
        :type id: int
        :param name: The name of the new security level
        :type name: str
        :returns: The newly created SecurityLevel
        :rtype: Optional[SecurityLevel]
        """

        data: dict[str, Any] = {"name": name}
        res = self.__ct.make_request(f"securitylevels/{id}", method="post", data=data)
        if res:
            return SecurityLevel(**res)

        return None

    def patch_security_level(
        self,
        id: int,
        name: str,
        forcereorder: bool | None = None,
        newid: int | None = None,
    ) -> SecurityLevel | None:
        """Change a security level

        :param id: The ID of the updated security level
        :type id: int
        :param name: The name of the updated security level
        :type name: str
        :param forcereorder: Needs to be true, if security level shall be reordered
        :type forcereorder: bool
        :param newid: The new ID of the updated security level
        :type newid: int
        :returns: The updated SecurityLevel
        :rtype: Optional[SecurityLevel]
        """

        params: dict[str, Any] = {}
        data: dict[str, Any] = {
            "name": name,
            "newid": newid if newid is not None else id,
        }

        if forcereorder is not None:
            params["forcereorder"] = forcereorder
            data["forcereorder"] = forcereorder

        res = self.__ct.make_request(
            f"securitylevels/{id}", params=params, method="patch", data=data
        )
        if res:
            return SecurityLevel(**res)

        return None

    def get_security_level(self, id: int) -> SecurityLevel | None:
        """Get a security level

        :param id: The ID of the updated security level
        :type id: int
        :returns: The SecurityLevel
        :rtype: Optional[SecurityLevel]
        """

        res = self.__ct.make_request(f"securitylevels/{id}")
        if res:
            return SecurityLevel(**res)

        return None

    def delete_security_level(self, id: int) -> bool:
        """Delete a security level

        :param id: The ID of the updated security level
        :type id: int
        :returns: The success
        :rtype: bool
        """

        res = self.__ct.make_request(
            f"securitylevels/{id}", method="delete", return_status_code=True
        )
        return res == 200



from __future__ import annotations

from pydantic import BaseModel


class Status(BaseModel):
    id: int
    name: str
    shorty: str
    isMember: bool
    isSearchable: bool
    sortKey: int
    securityLevelId: int



from __future__ import annotations

from pydantic import BaseModel


class Pagination(BaseModel):
    total: int
    current: int
    limit: int
    lastPage: int


class MetaPagination(BaseModel):
    count: int
    pagination: Pagination | None = None



from __future__ import annotations

from datetime import date, datetime

from pydantic import BaseModel, field_serializer, Field  # type: ignore

from . import EmptyStrToNone


class CalendarMeta(BaseModel):
    modifiedDate: date | datetime
    modifiedPid: int


class Calendar(BaseModel):
    campusId: int | None = None
    color: str
    eventTemplateId: int | None = None
    iCalSourceUrl: str | None = None
    id: int
    isPrivate: bool
    isPublic: bool
    meta: CalendarMeta | None = None
    name: str
    nameTranslated: str
    randomUrl: str
    sortKey: int


class Address(BaseModel):
    meetingAt: str | None = None
    street: str | None = None
    addition: str | None = None
    district: str | None = None
    zip: str | None = None
    city: str | None = None
    country: str | None = None
    latitude: EmptyStrToNone[float] | None = None
    longitude: EmptyStrToNone[float] | None = None


class AppointmentAddition(BaseModel):
    date: date | datetime
    isRepeated: bool | None = False


class AppointmentException(BaseModel):
    date: date | datetime
    id: int
    meta: CalendarMeta


class PersonWithOnlyId(BaseModel):
    id: int


class AppointmentMeta(BaseModel):
    createdDate: date | datetime
    createdPerson: PersonWithOnlyId
    modifiedDate: date | datetime
    modifiedPerson: PersonWithOnlyId


class Appointment(BaseModel):
    additions: list[AppointmentAddition] = Field(default_factory=list)
    address: Address | None = None
    allDay: bool = False
    calendar: Calendar | None = None
    caption: str
    endDate: date | datetime
    exceptions: list[AppointmentException] = Field(default_factory=list)
    id: int | str | None = None
    image: str | None = None
    information: str | None = None
    isInternal: bool = True
    link: str | None = None
    meta: AppointmentMeta | None = None
    note: str | None = None
    onBehalfOfPid: int | None = None
    startDate: date | datetime
    repeatFrequency: int | None = None
    repeatId: int = 0
    repeatOption: int | None = None
    repeatUntil: str | None = None
    version: int = 0

    @field_serializer("startDate")
    def serialize_startDate(self, dt: datetime, _info):
        return dt.strftime("%Y-%m-%dT%H:%M:%SZ")

    @field_serializer("endDate")
    def serialize_endDate(self, dt: datetime, _info):
        return dt.strftime("%Y-%m-%dT%H:%M:%SZ")



from __future__ import annotations

from typing import Any, Generic, TypeVar
from collections.abc import Generator

PydanticField = TypeVar("PydanticField")


class EmptyStrToNone(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> PydanticField | None:
        if v == "":
            return None
        return v


class EmptyStrToFalse(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> bool:
        if v == "" or not v:
            return False
        return True
